#include <algorithm>
#include <list>
#include <set>
#include <iterator>

#include "MeshLoader.h"

std::vector<Node>& MeshLoader::getNodes() {
    return nodes;   
}

std::vector<FiniteElement>& MeshLoader::getFiniteElements() {
    return elements;
}

std::vector<BoundaryFiniteElement>& MeshLoader::getBoundaryFiniteElements() {
    return surfaces;
}

std::vector<FiniteElement> MeshLoader::getElementsByID(const int id1, const int id2, const int id3) { 
    std::vector<FiniteElement> result;
    auto current_iter = elements.begin();
    auto last_iter = elements.end();
    while (current_iter != last_iter) {
        current_iter = std::find_if(current_iter, last_iter, [id1, id2, id3](FiniteElement current_element) {
            return std::find(current_element.nodesID.begin(), current_element.nodesID.end(), id1) != current_element.nodesID.end()
                && std::find(current_element.nodesID.begin(), current_element.nodesID.end(), id2) != current_element.nodesID.end()
                && std::find(current_element.nodesID.begin(), current_element.nodesID.end(), id3) != current_element.nodesID.end(); });
        if (current_iter != last_iter) {
            result.push_back(*current_iter);
            ++current_iter;
        }
    }
    return result;
}

std::vector<FiniteElement> MeshLoader::getElementsByEdge(const int id1, const int id2) {
    std::vector<FiniteElement> result;
    auto current_iter = elements.begin();
    auto last_iter = elements.end();
    while (current_iter != last_iter) {
        current_iter = std::find_if(current_iter, last_iter, [id1, id2](FiniteElement current_element) {
            return std::find(current_element.nodesID.begin(), current_element.nodesID.end(), id1) != current_element.nodesID.end()
                && std::find(current_element.nodesID.begin(), current_element.nodesID.end(), id2) != current_element.nodesID.end(); });
        if (current_iter != last_iter) {
            result.push_back(*current_iter);
            ++current_iter;
        }
    }
    return result;
}

std::vector<FiniteElement> MeshLoader::getElementsByOneID(const int id1) {
    std::vector<FiniteElement> result;
    auto current_iter = elements.begin();
    auto last_iter = elements.end();
    while (current_iter != last_iter) {
        current_iter = std::find_if(current_iter, last_iter, [id1](FiniteElement current_element) {
            return std::find(current_element.nodesID.begin(), current_element.nodesID.end(), id1) != current_element.nodesID.end(); });
        if (current_iter != last_iter) {
            result.push_back(*current_iter);
            ++current_iter;
        }
    }
    return result;
}

std::vector<Node> MeshLoader::getNodesByBoundary(int boundaryID) {
    std::set<Node> result;
    auto current_iter = surfaces.begin();
    auto last_iter = surfaces.end();
    while (current_iter != last_iter) {
        current_iter = std::find_if(current_iter, last_iter, [boundaryID](BoundaryFiniteElement current_element) {
            return current_element.boundaryID == boundaryID; });
        if (current_iter != last_iter) {
            for (const auto& it : current_iter->nodesID) {
                auto node_by_ID = std::find_if(nodes.begin(), nodes.end(), [it](Node n) { return n.nodeID == it; });
                result.insert(*node_by_ID);
            }
            ++current_iter;
        }
    }
    return std::vector<Node>(result.begin(), result.end());
}

std::vector<FiniteElement> MeshLoader::getElementsByBoundary(int boundaryID) {
    std::set<FiniteElement> result;
    auto current_iter = surfaces.begin();
    auto last_iter = surfaces.end();
    while (current_iter != last_iter) {
        current_iter = std::find_if(current_iter, last_iter, [boundaryID](BoundaryFiniteElement current_element) {
            return current_element.boundaryID == boundaryID; });
        if (current_iter != last_iter) {
            auto element_by_ID = std::find_if(elements.begin(), elements.end(), [current_iter](FiniteElement n) {
                return n.elementID == current_iter->elementID; });
            result.insert(*element_by_ID);
            ++current_iter;
        }
    }
    return std::vector<FiniteElement>(result.begin(), result.end());
}

std::vector<BoundaryFiniteElement> MeshLoader::getsSurfacesByBoundary(int boundaryID) {
    std::vector<BoundaryFiniteElement> result;
    auto current_iter = surfaces.begin();
    auto last_iter = surfaces.end();
    while (current_iter != last_iter) {
        current_iter = std::find_if(current_iter, last_iter, [boundaryID](BoundaryFiniteElement current_element) {
            return current_element.boundaryID == boundaryID; });
        if (current_iter != last_iter) {
            result.push_back(*current_iter);
            ++current_iter;
        }
    }
    return result;
}

void MeshLoader::insertCenterNode() {

}

/*std::vector<std::vector<Node>> MeshLoader::getNodeNeighbours() {
    std::set<Node> result_in;
    std::vector<std::vector<Node>> result_out;
    for (const auto& i : nodes) {
        for (const auto& j : getElementsByOneID(i.nodeID)) {
            for (auto k : j.nodesID) {
                auto node_by_ID = std::find_if(nodes.begin(), nodes.end(), [k](Node n) { return n.nodeID == k; });
                result_in.insert(*node_by_ID);
            }
        }
        result_out.push_back(std::vector<Node>(result_in.begin(), result_in.end()));
    }
    return result_out;
}*/

void MeshLoader::Print_Data() {
    std::cout << getNodes() << std::endl
        << getFiniteElements() << std::endl
        << getBoundaryFiniteElements() << std::endl;
}